import click

from compaipair.templates.cli_functions import (
    new_template,
    edit_template,
    show_templates,
)


@click.group(name="templates")
def templates_api():
    pass


@templates_api.command(name="new")
@click.argument("name")
@click.option(
    "--priming", "-p", default="", help="Text for priming this template's prompts"
)
@click.option(
    "--decorator", "-d", default="", help="Text for decorating this template's prompts"
)
def new_template_cli(name: str, priming, decorator):
    new_template(name, priming, decorator)


@templates_api.command("edit")
@click.argument("name")
@click.option(
    "--priming", "-p", default="", help="Text for priming this template's prompts"
)
@click.option(
    "--decorator", "-d", default="", help="Text for decorating this template's prompts"
)
def edit_template_cli(name: str, priming: str, decorator: str):
    edit_template(name, priming, decorator)


@templates_api.command("show", help="Show the list of available templates")
@click.option("--name", "-n", default=None, help="Name of the template to show.")
@click.option(
    "--verbose",
    "-v",
    default=False,
    is_flag=True,
    help="Show all the templates' contents",
)
def show_templates_cli(name, verbose):
    show_templates(name, verbose)



import google.generativeai as palm
from google.generativeai.types import Model, Completion

from compaipair.types.completion_template import CompletionTemplate
from compaipair.utils import configure_palm_api, get_available_models, get_api_key

prompt_template = """
{priming}
{question}
{decorator}
"""


class CompaiCompletion:
    model: Model
    priming: str
    question: str
    decorator: str
    result: Completion
    api_key: str
    template: CompletionTemplate | None

    def __init__(
        self,
        model: Model | str,
        template: str = None,
        question: str = "",
        priming: str = "",
        decorator: str = "",
        temperature: float = 0.7,
        api_key: str = None,
        input: str = None,
    ):
        self.temperature = temperature

        self.question = question
        if template is None:
            self.priming = priming
            self.decorator = decorator
        if template is not None:
            template = CompletionTemplate.find_template(template)
            self.priming = template.priming if template.priming else priming
            self.decorator = template.decorator if template.decorator else decorator

        if api_key is None:
            self.api_key = get_api_key()

        configure_palm_api(api_key=self.api_key)

        if isinstance(model, Model):
            self.model = model
        elif model is None:
            self.model = self.get_model()
        else:
            self.model = self.get_model(model)

    def complete(self) -> Completion:
        self.result = palm.generate_text(
            prompt=self.prompt, model=self.model, temperature=self.temperature
        )
        return self.result

    @staticmethod
    def get_model(model: str = "text-bison-001") -> Model:
        models = list(get_available_models())
        return next(filter(lambda m: model in m.name, models), models[1])

    @property
    def prompt(self):
        return prompt_template.format(
            priming=self.priming, question=self.question, decorator=self.decorator
        ).strip()



from rich.console import Console
from rich.markdown import Markdown

from compaipair.types.compaicompletion import CompaiCompletion
from compaipair.utils import configure_palm_api, get_available_models, close_db

console = Console()


available_models_output_template = """# {model_name}
- name: {model_name}
- description: {model_description}
- generation methods: 
{generation_methods}
---"""


def available_models():
    configure_palm_api()
    for m in get_available_models():
        generation_methods = "\n".join(
            [
                f"\t- {generation_method}"
                for generation_method in m.supported_generation_methods
            ]
        )
        output = available_models_output_template.format(
            model_name=m.name,
            model_description=m.description,
            generation_methods=generation_methods,
        )
        console.print(Markdown(output))


def complete(
    question: str,
    priming: str = "",
    decorator: str = "",
    model_name: str = None,
    temperature: float = 0.7,
    verbose: bool = False,
    plain_text_output: bool = False,
    template: str = None,
    output: str = None,
    input_file: str = None,
):
    if input_file is not None:
        print(f"Using input from file {input_file}")
        with open(input_file, "r") as f:
            question = f"{question}\n{f.read()}"

    completion = CompaiCompletion(
        question=question,
        priming=priming,
        decorator=decorator,
        model=model_name,
        temperature=temperature,
        template=template,
        input=input_file,
    )
    completion.complete()

    if completion.result.result is not None:
        completion_result = completion.result.result
        if verbose:
            completion_result = (
                f"# Prompt\n{completion.prompt}\n---\n# Result\n{completion_result}"
            )
        if not plain_text_output:
            completion_output = Markdown(completion_result)
        else:
            completion_output = completion_result
        console.print(completion_output)

        if output is not None:
            with open(output, "w") as f:
                f.write(completion_result)

    close_db()



import click

from compaipair.complete.cli_functions import complete, available_models


@click.command(name="complete")
@click.option("--priming", help="String to prime the LLM prompt")
@click.option("--decorator", help="Decorator to your prompt")
@click.option("-o", "--output", default="./result.md", help="File to write output")
@click.option(
    "-i",
    "--input-file",
    type=click.Path(exists=True, dir_okay=False),
    default=None,
    help="File with input to append to the question.",
)
@click.option(
    "-t",
    "--temperature",
    default=0.7,
    help="Model temperature, use 0 for more deterministic completions.",
)
@click.option(
    "-m", "--model-name", default="text-bison-001", help="LLM model for this query"
)
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    default=False,
    help="Verbose. Include prompt in output",
)
@click.option(
    "--plain-text",
    is_flag=True,
    default=False,
    help="Print output in plain text, otherwise will be formatted as Markdown",
)
@click.option(
    "--template",
    default=None,
    type=click.STRING,
    help="Template to prime and decorate prompt.",
)
@click.argument("question")
def complete_cli(
    question,
    priming,
    decorator,
    input_file,
    output,
    temperature,
    model_name,
    verbose,
    plain_text,
    template,
):
    complete(
        question=question,
        priming=priming,
        decorator=decorator,
        output=output,
        temperature=temperature,
        model_name=model_name,
        verbose=verbose,
        plain_text_output=plain_text,
        template=template,
        input_file=input_file,
    )


@click.command("complete-template")
@click.argument("template")
@click.option(
    "-q", "--question", help="Optional question that will be appended to the template."
)
@click.option("-o", "--output", default="./result.md", help="File to write output")
@click.option(
    "-i",
    "--input-file",
    type=click.Path(exists=True, dir_okay=False),
    default=None,
    help="File with input to append to the question.",
)
@click.option(
    "-t",
    "--temperature",
    default=0.7,
    help="Model temperature, use 0 for more deterministic completions.",
)
@click.option(
    "-m", "--model-name", default="text-bison-001", help="LLM model for this query"
)
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    default=False,
    help="Verbose. Include prompt in output",
)
@click.option(
    "--plain-text",
    is_flag=True,
    default=False,
    help="Print output in plain text, otherwise will be formatted as Markdown",
)
def complete_template_cli(
    question,
    priming,
    decorator,
    input_file,
    output,
    temperature,
    model_name,
    verbose,
    plain_text,
    template,
):
    complete(
        input_file=input_file,
        question=question,
        priming=priming,
        decorator=decorator,
        output=output,
        temperature=temperature,
        model_name=model_name,
        verbose=verbose,
        plain_text_output=plain_text,
        template=template,
    )


@click.command(name="available-models")
def available_models_cli():
    available_models()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['compaipair',
 'compaipair.complete',
 'compaipair.config',
 'compaipair.templates',
 'compaipair.types']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.7,<9.0.0',
 'google-generativeai>=0.2.2,<0.3.0',
 'pydantic>=2.4.2,<3.0.0',
 'pytest>=7.4.2,<8.0.0',
 'python-dotenv>=1.0.0,<2.0.0',
 'rich>=13.6.0,<14.0.0',
 'tinydb>=4.8.0,<5.0.0']

entry_points = \
{'console_scripts': ['compai = compaipair.main:compai']}

setup_kwargs = {
    'name': 'compaipair',
    'version': '0.4.0',
    'description': 'A simple CLI for pair programming with Google Generative AI APIs',
    'long_description': None,
    'author': 'Andrés Arango Pérez',
    'author_email': 'andresap@protonmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
